package GestureRecognition;

import PreTreatement.Quantizer;
import be.ac.ulg.montefiore.run.jahmm.*;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationSequencesReader;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationVectorReader;
import be.ac.ulg.montefiore.run.jahmm.learn.BaumWelchLearner;
import be.ac.ulg.montefiore.run.jahmm.learn.BaumWelchScaledLearner;
import be.ac.ulg.montefiore.run.jahmm.learn.KMeansLearner;

import java.io.FileReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

/**
 * User: mad - mad_bn@yahoo.com
 * Date: May 5, 2010
 * Time: 6:14:45 PM
 * PicFloyd
 */
public class GestureModel {
    private Hmm<ObservationInteger> hmm;
    private Quantizer quantizer;
    private double defaultProbability;
    private static int noStates = 8;
    private static int noOfFeatures = 3;
    private static int noOfObservations = 14;

    public GestureModel() {
        hmm = buildInitHMM(noStates);
    }

    Hmm<ObservationInteger> buildInitHMM(int noStates) {
        //ObservationVector is a vector of double
        Hmm<ObservationInteger> hmm = new Hmm<ObservationInteger>(noStates, new OpdfIntegerFactory(noOfObservations));
        for (int i = 0; i < noStates; i++) {
            hmm.setPi(i, 1.0 / (double) noStates);
            hmm.setOpdf(i, new OpdfInteger(noOfObservations));
        }

        for (int i = 0; i < noStates; i++)
            for (int j = 0; j < noStates; j++) {
                hmm.setAij(i, j, 1.0 / (double) noStates);
            }
        return hmm;
    }

    public void Learn(List<Gesture> learningSequences) {
        BaumWelchScaledLearner bwl = new BaumWelchScaledLearner();

        quantizer = new Quantizer(learningSequences);
        List<List<ObservationInteger>> seqsForHmm = new Vector<List<ObservationInteger>>();
        for (int i = 0; i < learningSequences.size(); i++) {
            Vector<ObservationInteger> gest;
            for (int j = 0; j < learningSequences.get(i).getData().size(); j++)
                System.out.print(learningSequences.get(i).getData().get(j).toString() + " ; ");
            System.out.println();
            gest = quantizer.getObservationSequence(learningSequences.get(i));
            for (int j = 0; j < gest.size(); j++)
                System.out.print(gest.get(j) + " ");
            System.out.println();
            seqsForHmm.add(gest);
        }

        /**
         * If we leave this initialization the algorithm will in fact behave poorly
         * because of an overfitting of the data
         *
         * KMeansLearner<ObservationInteger> kml =
                new KMeansLearner<ObservationInteger>(noStates,
                        new OpdfIntegerFactory(noOfObservations), seqsForHmm);
        hmm = kml.iterate();
         */
        for (int i = 0; i < 10; i++) {
            System.out.println(i + " ");
            System.out.println(hmm.toString());
            hmm = bwl.iterate(hmm, seqsForHmm);
        }
        System.out.println(hmm.toString());
        setDefaultProbability(seqsForHmm);
    }

    private void setDefaultProbability(List<List<ObservationInteger>> defsequence) {
        double prob = 0;
        for (int i = 0; i < defsequence.size(); i++) {
            prob += matches(defsequence.get(i));
        }

        defaultProbability = (prob) / defsequence.size();
    }


    public void Learn(String fileName) {
        try {
            Reader reader = new FileReader(fileName);
            List<List<ObservationVector>> seqs;
            seqs = ObservationSequencesReader.readSequences(new ObservationVectorReader(noOfFeatures), reader);
            List<Gesture> seqsGesture = new Vector<Gesture>();
            for (int i = 0; i < seqs.size(); i++) {
                Gesture gesture = new Gesture(seqs.get(i));
                seqsGesture.add(gesture);
            }

            Learn(seqsGesture);
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }


    public double matches(Gesture gesture) {
        List<ObservationInteger> obs = quantizer.getObservationSequence(gesture);
        return hmm.probability(obs);
    }

    public double matches(List<ObservationInteger> gesture) {
        return hmm.probability(gesture);
    }

    public double getDefaultProbability() {
        return defaultProbability;
    }
}
