package PreTreatement;

import GestureRecognition.Gesture;
import GestureRecognition.GestureEvent;
import be.ac.ulg.montefiore.run.jahmm.ObservationInteger;

import java.util.List;
import java.util.Vector;

/**
 * User: mad - mad_bn@yahoo.com
 * Date: May 7, 2010
 * Time: 9:47:15 AM
 * PicFloyd
 */
public class Quantizer {
    private double radius;
    private double[][] spherePoints = new double[14][3];

    public Quantizer(List<Gesture> gestures) {
        Vector<GestureEvent> all = new Vector<GestureEvent>();
        double maxAcc = 0, minAcc = 0;
        for (int i = 0; i < gestures.size(); i++) {
            Vector<GestureEvent> t = gestures.get(i).getData();
            maxAcc += gestures.get(i).getMaxAcceleration();
            minAcc += gestures.get(i).getMinAcceleration();

            for (int j = 0; j < gestures.get(i).getData().size(); j++) {
                all.add(t.elementAt(j));
            }
        }
        maxAcc /= gestures.size();
        minAcc /= gestures.size();
        initQuantizer(all, maxAcc, minAcc);
    }

    private void initQuantizer(Vector<GestureEvent> data, double maxAcc, double minAcc) {
        double pi = Math.PI;
        this.radius = (maxAcc + minAcc) / 2;

        spherePoints[0] = new double[]{radius, 0.0, 0.0};
        spherePoints[1] = new double[]{Math.cos(pi / 4) * radius, Math.sin(pi / 4) * radius, 0.0};
        spherePoints[2] = new double[]{0.0, radius, 0.0};
        spherePoints[3] = new double[]{-Math.cos(pi / 4) * radius, Math.sin(pi / 4) * radius, 0.0};
        spherePoints[4] = new double[]{-radius, 0.0, 0.0};
        spherePoints[5] = new double[]{-Math.cos(pi / 4) * radius, -Math.sin(pi / 4) * radius, 0.0};
        spherePoints[6] = new double[]{0.0, -radius, 0.0};
        spherePoints[7] = new double[]{Math.cos(pi / 4) * radius, -Math.sin(pi / 4) * radius, 0.0};

        spherePoints[8] = new double[]{Math.cos(pi / 4) * radius, 0.0, Math.sin(pi / 4) * radius};
        spherePoints[9] = new double[]{0.0, 0.0, radius};
        spherePoints[10] = new double[]{-Math.cos(pi / 4) * radius, 0.0, Math.sin(pi / 4) * radius};
        spherePoints[11] = new double[]{-Math.cos(pi / 4) * radius, 0.0, -Math.sin(pi / 4) * radius};
        spherePoints[12] = new double[]{0.0, 0.0, -radius};
        spherePoints[13] = new double[]{Math.cos(pi / 4) * radius, 0.0, -Math.sin(pi / 4) * radius};

        findDirectionVectors(data);
    }

    private int[][] copyArray(int[][] a) {
        int[][] neu = new int[a.length][a[0].length];
        for (int i = 0; i < a.length; i++) {
            for (int j = 0; j < a[i].length; j++) {
                neu[i][j] = a[i][j];
            }
        }
        return neu;
    }

    private boolean arrayEquals(int[][] a, int[][] b) {
        for (int i = 0; i < a.length; i++) {
            for (int j = 0; j < a[i].length; j++) {
                if (!(a[i][j] == b[i][j])) {
                    return false;
                }
            }
        }
        return true;
    }

    public void findDirectionVectors(Vector<GestureEvent> data) {
        int[][] groups = new int[spherePoints.length][data.size()];
        int[][] groupsOld;
        do {
            groupsOld = this.copyArray(groups);
            groups = this.deriveGroups(data);
            double xMean, yMean, zMean;

            for (int i = 0; i < this.spherePoints.length; i++) {
                xMean = 0;
                yMean = 0;
                zMean = 0;
                int noOfElements = 0;
                for (int j = 0; j < data.size(); j++) {
                    if (groups[i][j] == 1) {
                        xMean += data.elementAt(j).getXAcc();
                        yMean += data.elementAt(j).getYAcc();
                        zMean += data.elementAt(j).getZAcc();
                        noOfElements++;
                    }
                }
                if (noOfElements > 1) {
                    this.spherePoints[i][0] = (xMean / (double) noOfElements);
                    this.spherePoints[i][1] = (yMean / (double) noOfElements);
                    this.spherePoints[i][2] = (zMean / (double) noOfElements);
                }
            }
        } while (!arrayEquals(groupsOld, groups));

    }

    public int[][] deriveGroups(Vector<GestureEvent> data) {
        int[][] groups = new int[spherePoints.length][data.size()];
        double[] currAcceleration;
        double[] diff = new double[3];
        double currentError;

        for (int j = 0; j < data.size(); j++) {
            double smallestError = Double.MAX_VALUE;
            int row = 0;
            currAcceleration = data.elementAt(j).getAcceleration();
            for (int i = 0; i < this.spherePoints.length; i++) {
                groups[i][j] = 0;
                double[] sphereP = spherePoints[i];
                diff[0] = sphereP[0] - currAcceleration[0];
                diff[1] = sphereP[1] - currAcceleration[1];
                diff[2] = sphereP[2] - currAcceleration[2];
                currentError = Math.sqrt((diff[0] * diff[0])
                        + (diff[1] * diff[1]) + (diff[2] * diff[2]));
                if (currentError < smallestError) {
                    smallestError = currentError;
                    row = i;
                }
            }
            groups[row][j] = 1;
        }
        return groups;
    }

    public Vector<ObservationInteger> getObservationSequence(Gesture gesture) {
        int[][] groups = this.deriveGroups(gesture.getData());
        Vector<ObservationInteger> sequence = new Vector<ObservationInteger>();

        for (int j = 0; j < groups[0].length; j++) {
            for (int i = 0; i < groups.length; i++) {
                if (groups[i][j] == 1) {
                    sequence.add(new ObservationInteger(i));
                    break;
                }
            }
        }

        while (sequence.size() < spherePoints.length) {
        	sequence.add(sequence.elementAt(sequence.size() - 1));
        }

        return sequence;
    }

}
