package WiiRemote;


import GestureRecognition.ClassifierManager;
import be.ac.ulg.montefiore.run.jahmm.ObservationVector;
import wiiremotej.WiiRemote;
import wiiremotej.WiiRemoteExtension;
import wiiremotej.WiiRemoteJ;
import wiiremotej.WiiRemotePassthroughExtension;
import wiiremotej.event.*;

import java.io.BufferedWriter;
import java.text.DecimalFormat;
import java.util.Vector;

import static java.lang.Thread.sleep;


public class WiiDataRetriever implements WiiRemoteListener {
    private WiiRemote remote;
    private String buttonPressed = "";
    private String gesture = "";
    private static BufferedWriter bw = null;
    private static double dataNo = 0;
    private static long begin = 0;
    private static long end = 0;
    private static double currentNo = 0;
    private double lastX, lastY, lastZ;
    private double lasts[] = new double[9];
    private Boolean readAcceleration = false;
    private Vector<ObservationVector> observations;

    public static void init() {
        System.setProperty("bluecove.jsr82.psm_minimum_off", "true");
        WiiRemoteJ.setConsoleLoggingAll();
        //Find and connect to a Wii Remote
        try {
            WiiRemote remote = null;
            int count = 0;
            while (remote == null) {
                try {
                    remote = WiiRemoteJ.findRemote();
                }
                catch (Exception e) {

                    System.out.println(count + e.getMessage());
                    count++;
                    if (remote != null)
                        System.out.println("Not NULL: " + remote.isConnected());
                    System.out.println("Failed to connect remote. Trying again.");
                    sleep(1000);
                    remote = null;
                }
            }
            remote.addWiiRemoteListener(new WiiDataRetriever(remote));
            remote.setAccelerometerEnabled(true);
            remote.setSpeakerEnabled(true);
            remote.setIRSensorEnabled(true, WRIREvent.BASIC);
            remote.setLEDIlluminated(0, true);
            try {
                remote.setExtensionEnabled(true);
            } catch (Exception e) {
                System.out.println(e.getMessage());
            }

            final WiiRemote remoteF = remote;
            Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
                public void run() {
                    remoteF.disconnect();
                }
            }));
        }
        catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    public WiiDataRetriever(WiiRemote remote) {
        this.remote = remote;
        observations = new Vector<ObservationVector>();
        ClassifierManager.getInstance().hmmTest("src/TestFiles/squareTesting.txt", 1);
    }

    public void disconnected() {
        System.out.println("Remote disconnected... Please Wii again.");
        System.exit(0);
    }

    public void statusReported(WRStatusEvent evt) {
        System.out.println("Battery level: " + (double) evt.getBatteryLevel() / 2 + "%");
        System.out.println("Continuous: " + evt.isContinuousEnabled());
        System.out.println("Remote continuous: " + remote.isContinuousEnabled());
    }

    public void IRInputReceived(WRIREvent evt) {
        /*for (IRLight light : evt.getIRLights())
        {
            if (light != null)
            {
                System.out.println("X: "+light.getX());
                System.out.println("Y: "+light.getY());
            }
        }*/

    }

    public void accelerationInputReceived(WRAccelerationEvent evt) {
        try {
            if (readAcceleration)

                if (buttonPressed.equalsIgnoreCase("A")) {
                    DecimalFormat df = new DecimalFormat("0.00");

                    double cX = evt.getXAcceleration();
                    double cY = evt.getYAcceleration();
                    double cZ = evt.getZAcceleration();
                    /*if (lastX==cX && lastY==cY && lastZ==cZ)
                        System.out.println("Error");*/

                    double meanX = (cX + lastX) / 2;
                    double meanY = (cY + lastY) / 2;
                    double meanZ = (cZ + lastZ) / 2;

                    lastX = cX;
                    lastY = cY;
                    lastZ = cZ;


                    dataNo++;

                    if (dataNo % 3 == 0) {
                        String X = df.format(meanX);
                        String Y = df.format(meanY);
                        String Z = df.format(meanZ);
                        double[] acc = {Double.parseDouble(X), Double.parseDouble(Y), Double.parseDouble(Z)};
                        ObservationVector obs = new ObservationVector(acc);
                        observations.add(obs);
                    }

                }
        }
        catch (Exception e) {
        }


    }

    public void extensionInputReceived(WRExtensionEvent evt) {
        System.out.println("Extension Input Received");
        if (evt instanceof WRNunchukExtensionEvent) {
            System.out.println("WR Nunchuck Extension");

        } else if (evt instanceof WRClassicControllerExtensionEvent) {
            System.out.println("WR Classic Controller");
        } else if (evt instanceof WRGuitarExtensionEvent) {
            System.out.println("WR Guitar Extension");
        }
    }

    private byte[] intToByteArray(final int integer) {
        int byteNum = (40 - Integer.numberOfLeadingZeros(integer < 0 ? ~integer : integer)) / 8;
        byte[] byteArray = new byte[4];

        for (int n = 0; n < byteNum; n++)
            byteArray[3 - n] = (byte) (integer >>> (n * 8));

        return (byteArray);
    }

    private byte[] byteToByteArray(final byte no) {
        byte[] b = new byte[1];
        b[0] = no;
        return b;
    }

    public void extensionConnected(WiiRemoteExtension extension) {
        System.out.println("1. Extension connected!");

        //remote.getExtensionOffset()

        WiiRemoteExtension w = remote.getExtension();

        WiiRemotePassthroughExtension wiiExtension = remote.getPassthroughExtension();
        long codep = wiiExtension.getPassthroughCode();
        System.out.println("Passthrough Code: " + codep + " ExtOffset " + remote.getExtensionOffset() + " ExtOffset " + Integer.toHexString(remote.getExtensionOffset()));
        System.out.println("EXCon: " + remote.isExtensionConnected() + " ExEnab " + remote.isExtensionEnabled());


        //The extension is initialised by writing 0x55 to 0x(4)a600f0.

        byte writeBInit1 = 0x55;
        int addressToWriteInit = 0x4A600F0;

        try {
            try {
                remote.writeData(intToByteArray(addressToWriteInit), byteToByteArray(writeBInit1));

            }
            catch (Exception ex) {
                System.out.println(" exception " + ex.getMessage());
            }
            sleep(1000);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        //While the Motion Plus is initialising, the value at 0x(4)a600f7 changes
        // from 0x02 to 0x04 to 0x08 to 0x0C to 0x0E then stays at 0x0E.
        int addressTest3 = 0x4a600f7;
        try {
            for (int i = 0; i < 10; i++) {
                try {
                    byte[] readData = remote.readData(intToByteArray(addressTest3), 2);
                    System.out.println("Read Data Initialising: ");
                    for (int j = 0; j < readData.length; j++)
                        System.out.print(Integer.toHexString(readData[j]));
                }
                catch (Exception ex) {
                    System.out.println(i + " exception " + ex.getMessage());
                }
                sleep(1000);
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }

        //Writing 0x04 to 0x(4)A600FE activates the MotionPlus as the "active" extension.
        //activate the Motion Plus as the active extention
        System.out.println("Writing 0x04 to 0x(4)A600FE activates the MotionPlus as the \"active\" extension.");
        byte writeBInit = 0x04;
        int addressToWrite1 = 0x4A600FE;

        try {
            try {
                remote.writeData(intToByteArray(addressToWrite1), byteToByteArray(writeBInit));

            }
            catch (Exception ex) {
                System.out.println("!!! exception !!!" + ex.getMessage());
            }
            sleep(1000);
        }
        catch (Exception e) {
            e.printStackTrace();
        }


        try {
            try {
                byte[] readData = remote.readData(intToByteArray(0x4A600FE), 1);
                System.out.println("Read Data 0x4A600FE: ");
                for (int j = 0; j < readData.length; j++)
                    System.out.print(Integer.toHexString(readData[j]));
            }
            catch (Exception ex) {
                System.out.println("!!! exception: !!! " + ex.getMessage());
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }

        System.out.println("The standard extension identifier at 0x(4)A400FA now reads 00 00 A4 20 04 05");
        int addressTest2 = 0x4A400FA;

        try {
            try {
                byte[] readData = remote.readData(intToByteArray(addressTest2), 6);
                System.out.println("Read Data: ");
                for (int j = 0; j < readData.length; j++)
                    System.out.print(Integer.toHexString(readData[j]));
            }
            catch (Exception ex) {
                System.out.println("!!! exception: !!! " + ex.getMessage());
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }


/*          */
    }

    public void extensionPartiallyInserted() {
        System.out.println("Extension partially inserted. Push it in more next time, jerk!");
    }

    public void combinedInputReceived(wiiremotej.event.WRCombinedEvent wrCombinedEvent) {

    }

    public void extensionUnknown() {
        System.out.println("Extension unknown. Did you try to plug in a toaster or something?");
    }

    public void extensionDisconnected(WiiRemoteExtension extension) {
        System.out.println("Extension disconnected. Why'd you unplug it, retard?");
    }

    private void mouseCycle() {
        System.out.println("Mouse Cycle");
    }

    public void buttonInputReceived(WRButtonEvent evt) {

        if (evt.wasPressed(WRButtonEvent.TWO)) System.out.println("2");
        if (evt.wasPressed(WRButtonEvent.ONE)) System.out.println("1");
        if (evt.wasPressed(WRButtonEvent.B)) {
            System.out.println("B");
            buttonPressed = "B";
            end = System.currentTimeMillis();
            long duration = end - begin;
            double sec = (double) duration / (double) 1000;
            double freq = dataNo / sec;
            System.out.println(" B: " + begin + " E: " + end + " M: " + duration + " No " + dataNo + " freq = " + freq);
            System.out.println("No of training data " + currentNo);
            currentNo++;
            dataNo = 0;
            begin = 0;
            end = 0;
            readAcceleration = false;
            System.out.println("Recognized gesture: " + ClassifierManager.getInstance().classify(observations));
            observations.clear();
        }
        if (evt.wasPressed(WRButtonEvent.A)) {
            System.out.println("A");
            buttonPressed = "A";
            begin = System.currentTimeMillis();
            readAcceleration = true;

        }
        if (evt.wasPressed(WRButtonEvent.MINUS)) System.out.println("Minus");
        if (evt.wasPressed(WRButtonEvent.HOME)) System.out.println("Home");
        if (evt.wasPressed(WRButtonEvent.LEFT)) System.out.println("Left");
        if (evt.wasPressed(WRButtonEvent.RIGHT)) System.out.println("Right");
        if (evt.wasPressed(WRButtonEvent.DOWN)) System.out.println("Down");
        if (evt.wasPressed(WRButtonEvent.UP)) System.out.println("Up");
        if (evt.wasPressed(WRButtonEvent.PLUS)) System.out.println("Plus");
    }

}
